"""SMU Model 2401 device driver module."""
from tm_devices.drivers.pi.source_measure_units.smu2400.smu2400_standard import SMU2400Standard


class SMU2401(SMU2400Standard):
    """SMU2401 device driver."""
